/*
 * em847x.c: Implementation of the SigmaDesign EM847x On Screen Display
 *
 * See the README file for copyright information and how to reach the author.
 *
 * $Id$
 */

#include "em847x.h"

cEm847x::cEm847x() {
	// printf("cEm847x::cEm847x\n");
	emDev=RUA_OpenDevice(0,FALSE);
	if (emDev==NULL) {
		esyslog("Cannot access SigmaDevice\n");
		exit(1);
	}
	vidbuf = (unsigned char *)malloc(sizeof(unsigned char)*1024*1024);
	vidlen = 0;
	cur_pts=0;
	cur_dts=0;
	RUA_DECODER_STREAMSTOP(emDev,REALMAGICHWL_VIDEO|REALMAGICHWL_SPU|REALMAGICHWL_OSD|REALMAGICHWL_AUDIO);

	// init de la video et de l'audio, a mettre ailleurs ...
	video_type = edecVideoStd_MPEG12Video;
	RUA_DECODER_SET_PROPERTY(emDev,DECODER_SET,edecVideoStd,sizeof(edecVideoStd_type),&video_type);
	standard_type = evTvStandard_PAL;
	RUA_DECODER_SET_PROPERTY(emDev,VIDEO_SET,evTvStandard,sizeof(evTvStandard_type),&standard_type);
	inputstandard_type = evInStandard_PAL;

	PCR_INFO pcr_time_info;
	pcr_time_info.PcrEnable=0;
	pcr_time_info.DeltaSCR_APTS_Max=180;
	pcr_time_info.TimeResolution=90000;
	RUA_DECODER_SET_PROPERTY(emDev,TIME_SET,etimPcrInfo,sizeof(PCR_INFO),&pcr_time_info);
	RUA_DECODER_SET_PROPERTY(emDev,VIDEO_SET,evInStandard,sizeof(evInStandard_type),&inputstandard_type);
	RUA_DECODER_GET_PROPERTY(emDev,VIDEO_SET,evMaxDisplayWindow,sizeof(Wnd_type),&source_window);
	printf("Max window Dest {%d,%d,%d,%d}\n",source_window.x,source_window.y,source_window.w,source_window.h);
	RUA_DECODER_SET_PROPERTY(emDev,VIDEO_SET,evDestinationWindow,sizeof(Wnd_type),&source_window);
       	speed = 256;
       	RUA_DECODER_SET_PROPERTY(emDev,VIDEO_SET,evSpeed,sizeof(evSpeed_type),&speed);
	broadcast_mode = 1;
	RUA_DECODER_SET_PROPERTY(emDev,VIDEO_SET,evBroadcastedVideo,sizeof(int),&broadcast_mode);
	eAudioFormat_type audio_format = eAudioFormat_MPEG2;
	RUA_DECODER_SET_PROPERTY(emDev,AUDIO_SET,eAudioFormat,sizeof(eAudioFormat_type),&audio_format);
	eAudioMode_type audio_mode = eAudioMode_Stereo;
	RUA_DECODER_SET_PROPERTY(emDev,AUDIO_SET,eAudioMode,sizeof(eAudioMode_type),&audio_mode);
	RMuint32 audio_channels = 2;
	RUA_DECODER_SET_PROPERTY(emDev,AUDIO_SET,eAudioNumberOfChannels,sizeof(RMuint32),&audio_channels);
	RMuint32 audio_bits = 16;
	RUA_DECODER_SET_PROPERTY(emDev,AUDIO_SET,eAudioNumberOfBitsPerSample,sizeof(RMuint32),&audio_bits);
	RMuint32 audio_samplerate = 48000;
	RUA_DECODER_SET_PROPERTY(emDev,AUDIO_SET,eAudioSampleRate,sizeof(RMuint32),&audio_samplerate);
	
	// fin init
	
	InitOsd(720, 576, 8, 0);
	RUA_DECODER_STREAMPLAY(emDev,REALMAGICHWL_VIDEO|REALMAGICHWL_SPU|REALMAGICHWL_OSD|REALMAGICHWL_AUDIO, RUA_VideoHwPlayNormal, 0);
	pOsd=(unsigned char *)mmap(0, osdDim.width*osdDim.height/(8/osdDim.color_depth), PROT_READ|PROT_WRITE, MAP_SHARED, (int)emDev, REALMAGICHWL_MMAP_OSDBUF);
	ClearOsd();
#ifdef TVIA
	MapTVIAMemory();
	Tvia_DoSyncLock(PORTA, PAL, PAL720x576, CCIR656);
#endif
	esyslog("SigmaDevice access on %d\n",(int)emDev);
	// printf("cEm847x::cEm847x END\n");
}

cEm847x::~cEm847x() {
	// printf("cEm847x::~cEm847x\n");
	RUA_DECODER_STREAMSTOP(emDev,REALMAGICHWL_VIDEO|REALMAGICHWL_SPU|REALMAGICHWL_OSD|REALMAGICHWL_AUDIO);
    	munmap(pOsd, osdDim.width*osdDim.height/(8/osdDim.color_depth));
#ifdef TVIA
	Tvia_CleanupSyncLock();
#endif
	RUA_ReleaseDevice(emDev);
	esyslog("SigmaDevice %d destroyed\n",(int)emDev);
	// printf("cEm847x::~cEm847x END\n");
}

void cEm847x::InitOsd(RMuint32 width, RMuint32 height, int depth, RMbool rlc) {
	// printf("cEm847x::InitOsd\n");

	SetOsdActive(0);
	osd_command=eOsdOff;
	RUA_DECODER_SET_PROPERTY(emDev, OSD_SET, eOsdCommand, sizeof(eOsdCommand_type), &osd_command);
	osdDim.width=width;
	osdDim.height=height;
	osdDim.color_depth=OSDBUF_8BPP;
	osdDim.rlc=0; 
        Kp.kernelpropid_value=PROPID_KERNEL_OSDBUF_Dimensions;
        Kp.PropTypeLength=sizeof(RMKERNELOSDBUFDimensions_type);
        Kp.pValue=&osdDim;
        if(RUA_KERNEL_SET_PROPERTY(emDev,&Kp) !=0) {
	  // printf("Error while resizing\n");
        }
	else
	{
	  // printf("Resize OK\n");
	}
	SetOsdActive(1);
	SetOsdDim();
	Wnd_type w;
	w.w=osdDim.width;
	w.h=osdDim.height;
	RUA_DECODER_SET_PROPERTY(emDev, OSD_SET, eOsdDestinationWindow, sizeof(Wnd_type), &w);
	for (int i = 0 ; i <= 255 ; i++)
	{
		SetOsdPalette(i, 0, 0, 0, 0);
	}
	// printf("cEm847x::InitOsd END\n");
}

void cEm847x::SetOsdPalette(int i, char r, char g, char b, char alpha) {
	// printf("cEm847x::SetOsdPalette\n");
	RMKERNELOSDBUFPaletteEntry_type color;
	color.i = i;
	color.R = r;
	color.G = g;
	color.B = b;
	color.alpha = alpha;
	Kp.kernelpropid_value=PROPID_KERNEL_OSDBUF_PaletteEntry;
	Kp.PropTypeLength=sizeof(RMKERNELOSDBUFPaletteEntry_type);
	Kp.pValue=&color;
	RUA_KERNEL_SET_PROPERTY(emDev,&Kp);
	// printf("cEm847x::SetOsdPalette END\n");
}
void cEm847x::SetOsdDim() {
	// printf("cEm847x::SetOsdDim\n");
	Kp.kernelpropid_value=PROPID_KERNEL_OSDBUF_Dimensions;
	Kp.PropTypeLength=sizeof(RMKERNELOSDBUFDimensions_type);
	Kp.pValue=&osdDim;
	RUA_KERNEL_GET_PROPERTY(emDev,&Kp);
	// printf("cEm847x::SetOsdDim END\n");
}
void cEm847x::SetOsdActive(int i) {
	// printf("cEm847x::SetOsdActive\n");
	RMKERNELOSDBUFActivate_type RMKERNELOSDBUFActivate_value;
	if(i==0)
		RMKERNELOSDBUFActivate_value=FALSE;
	else
		RMKERNELOSDBUFActivate_value=TRUE;
	Kp.kernelpropid_value=PROPID_KERNEL_OSDBUF_Activate;
	Kp.PropTypeLength=sizeof(RMKERNELOSDBUFActivate_type);
	Kp.pValue=&RMKERNELOSDBUFActivate_value;
	RUA_KERNEL_SET_PROPERTY(emDev,&Kp);
	// printf("cEm847x::SetOsdActive END\n");
}
void cEm847x::SetOsdOn(int i) {
	// printf("cEm847x::SetOsdOn\n");
	if ((i>0) && (osd_command == eOsdOff))
	{
		osd_command=eOsdOn;
		RUA_DECODER_SET_PROPERTY(emDev, OSD_SET, eOsdCommand, sizeof(eOsdCommand_type), &osd_command);
	}
	else
	{
		if(osd_command == eOsdOn) {
			osd_command=eOsdOff;
			RUA_DECODER_SET_PROPERTY(emDev, OSD_SET, eOsdCommand, sizeof(eOsdCommand_type), &osd_command);
		}
	}
	// printf("cEm847x::SetOsdOn END\n");
}
void cEm847x::SetOsdPixel(RMuint32 X, RMuint32 Y, RMuint8 IDX) {
              // em847x->SetOsdPixel(X,Y,IDX);
              OSD_SetPixel(pOsd, &osdDim, X, Y, IDX);
}
void cEm847x::ClearOsd() {
	// Penser  patcher en fonction des dimensions OSD
	memset(pOsd,0xFF,720*576*sizeof(unsigned char));
}
void cEm847x::DrawOsd(unsigned char* buf) {
	// Penser  patcher en fonction des dimensions OSD
	memcpy(pOsd,buf,720*576*sizeof(unsigned char));
}
void cEm847x::SetVideoFormat(bool VideoFormat16_9) {
	// voir evInAspectRatio et evOutDisplayOption
}
void cEm847x::SetAudioFormat() {
}
void cEm847x::StopVideo() {
	if(videoActive)
	{
		RUA_DECODER_STREAMSTOP(emDev,REALMAGICHWL_VIDEO|REALMAGICHWL_SPU|REALMAGICHWL_AUDIO);
		// RUA_DECODER_STREAMSTOP(emDev,REALMAGICHWL_AUDIO);
		// RUA_DECODER_SET_PROPERTY(emDev,VIDEO_SET,evVideoHwStop);
		// RUA_DECODER_SET_PROPERTY(emDev, OSD_SET, eOsdCommand, sizeof(eOsdCommand_type), &osd_command);
		videoActive=false;
		vidlen=0;
	}
}
void cEm847x::PauseVideo() {
	if(videoActive)
	{
		RUA_DECODER_STREAMPAUSE(emDev,REALMAGICHWL_VIDEO|REALMAGICHWL_SPU|REALMAGICHWL_AUDIO);
		videoActive=false;
	}
}
void cEm847x::StartVideo() {
	if(!videoActive)
	{
		// RUA_DECODER_SET_PROPERTY(emDev,VIDEO_SET,evVideoHwPlay);
		RUA_DECODER_STREAMPLAY(emDev,REALMAGICHWL_VIDEO|REALMAGICHWL_SPU|REALMAGICHWL_AUDIO, RUA_VideoHwPlayNormal, 0);
		// RUA_DECODER_STREAMPLAY(emDev,REALMAGICHWL_AUDIO, 0, 0);

		videoActive=true;
		vidlen=0;
	}
}

void cEm847x::PlayAudio(RMuint8 *Data, RMuint32 Length, RMuint64 pts, RMuint64 dts, RMuint64 scr) {
//	printf("Received Audio Packet len : %d / PTS=%llu / DTS=%llu\n",Length,pts,dts);
        // unsigned char* buf = (unsigned char *)malloc(sizeof(unsigned char)*Length);
	// memcpy(buf,Data,Length);
                int flags=0;
                RMuint32 X=0;
                my_MPEG_WRITE_DATA Q={0,};
                Q.pData=Data;
                // Q.pData=buf;
                Q.DataLeft=Length;
		if(scr!=0) { SetStc(scr); }
                if(pts!=0)
                {
                        flags = RUA_PTS_AVAILABLE_FLAG;
                        // Q.Pts=CalcTs(pts);
                        Q.Pts=pts;
//                      printf("Str PTS : %llu [%llu]",Q.Pts,pts);
                }
#define COPYANDQUEUEDATA_TIMEOUT_MICROSEC 100000
                int cpt=0;
		while (RUA_DECODER_FEEDME(emDev,REALMAGICHWL_AUDIO,flags,&Q)!=0) {
			cpt++;
			X=REALMAGICHWL_HAPPENING_DECODER_AUDIODMA_HALF;
			RUA_DECODER_WAIT(emDev,COPYANDQUEUEDATA_TIMEOUT_MICROSEC,&X);
			if(cpt==3) break;
		}
}
void cEm847x::PlayVideo(RMuint8 *Data, RMuint32 Length, RMuint64 pts, RMuint64 dts, RMuint64 scr) {
bool play=false;
bool newes=false;
	if(pts==0) {
		// remplir le buffer
		memcpy(vidbuf+vidlen,Data,Length);
		vidlen = vidlen+Length;
	}
	else
	{
		play = true;
		newes = true;
	}

	if (play) {
		// play curpts,curdts, vidbuf,vidlen
		int flags=0;
		RMuint32 X=0;
		my_MPEG_WRITE_DATA Q={0,};
		Q.pData=vidbuf;
		Q.DataLeft=vidlen;
		if(cur_pts!=0) 
		{
			flags = RUA_PTS_AVAILABLE_FLAG;
			// Q.Pts=CalcTs(pts);
			Q.Pts=cur_pts;
		}
#define COPYANDQUEUEDATA_TIMEOUT_MICROSEC 100000
		int cpt=0;
		while (RUA_DECODER_FEEDME(emDev,REALMAGICHWL_VIDEO,flags,&Q)!=0) {
			cpt++;
			X=REALMAGICHWL_HAPPENING_DECODER_VIDEODMA_HALF;
			RUA_DECODER_WAIT(emDev,COPYANDQUEUEDATA_TIMEOUT_MICROSEC,&X);
			// if(cpt==3) break;
		}

		vidlen=0;
		cur_pts=0;
		cur_dts=0;
	}

	if (newes) {
		cur_pts=pts;
		cur_dts=dts;
		memcpy(vidbuf,Data,Length);
		vidlen=Length;
		
	}



}
void cEm847x::SetStc(RMuint64 scr) { 
	systemClock = scr;
	systemClockSign = true;
}
RMuint64 cEm847x::CalcTs(RMuint64 ts) {
	if(systemClockSign)
		return (ts-systemClock);
	else
		return (ts+systemClock);
}
RMuint64 cEm847x::GetStc(void) { 
        etimSystemTimeClock_type stc_time;
        RUA_DECODER_GET_PROPERTY(emDev, TIME_SET, etimSystemTimeClock, sizeof(etimSystemTimeClock_type), &stc_time);
	return (stc_time.dwlTime );
}
void cEm847x::PrintStc() {
	etimSystemTimeClock_type pts_time;
	etimSystemTimeClock_type dts_time;
	etimSystemTimeClock_type stc_time;
	printf("Sim STC Offset = %llu, Direction =", systemClock);
	if(systemClockSign) printf(" STC < PCR\n"); 
	else printf("STC > PCR\n");
	RUA_DECODER_GET_PROPERTY(emDev, TIME_SET, etimSystemTimeClock, sizeof(etimSystemTimeClock_type), &stc_time);
	printf("STC : %llu",stc_time.dwlTime);
	RUA_DECODER_GET_PROPERTY(emDev, TIME_SET, etimVideoFrameDisplayedTime, sizeof(etimSystemTimeClock_type), &pts_time);
	printf(" - PTS : %llu",pts_time.dwlTime);
	RUA_DECODER_GET_PROPERTY(emDev, TIME_SET, etimLastDecodedVideoFrameTime, sizeof(etimSystemTimeClock_type), &dts_time);
	printf(" - DTS : %llu\n",dts_time.dwlTime);


}
