/*
 * em847x.h: Implementation of the SigmaDesign EM847x On Screen Display
 *
 * See the README file for copyright information and how to reach the author.
 *
 * $Id$
 */
#ifndef EM847X_H
#define EM847X_H
#include <vdr/tools.h>
#include <sys/mman.h>
#include <stdlib.h>

#ifdef TVIA
#include "tvia/global.h"
extern "C" {
#include "tvia/synclock.h"
}
#include "tvia/igsiomem.h" /*low level h/w programming*/
#include "tvia/utility.h" /*TVIA utility library header file*/
#include "tvia/mapmem.h"
#endif //TVIA

#define ALLOW_OS_CODE 1
#include "include/realmagichwl_uk.h"


class cEm847x {
	private:
		bool videoActive;
		RUA_handle emDev;
		edecVideoStd_type video_type;
		evTvStandard_type standard_type;
		int broadcast_mode;
		evInStandard_type inputstandard_type;
		Wnd_type source_window;
       		evSpeed_type speed;
		eOsdCommand_type osd_command;
		kernelproperty Kp;
		RMKERNELOSDBUFDimensions_type osdDim;
		unsigned char *pOsd;
		unsigned char* vidbuf;
		RMuint32 vidlen;
		RMuint64 cur_pts;
		RMuint64 cur_dts;

		bool OSDpresent;
		RMuint64 systemClock;
		bool systemClockSign;
		void InitOsd(RMuint32 width, RMuint32 height, int depth, RMbool rlc);
		void SetOsdDim(void);
		void SetOsdActive(int i);
	public:
		cEm847x();
		~cEm847x();
		void SetOsdPalette(int i, char r, char g, char b, char alpha);
		void SetOsdOn(int i);
		void ClearOsd(void);
		void SetOsdPixel(RMuint32 X, RMuint32 Y, RMuint8 IDX);
		void DrawOsd(unsigned char* buf);
		void SetVideoFormat(bool VideoFormat16_9);
		void SetAudioFormat(void);
		void PlayVideo(RMuint8 *Data, RMuint32 Length, RMuint64 pts, RMuint64 dts, RMuint64 scr);
		void PlayAudio(RMuint8 *Data, RMuint32 Length, RMuint64 pts, RMuint64 dts, RMuint64 scr);
		void StartVideo(void);
		void StopVideo(void);
		void PauseVideo(void);
		void SetStc(RMuint64 stc);
		RMuint64 GetStc(void);
		RMuint64 CalcTs(RMuint64 ts);
		void PrintStc(void);
}; 

#endif // EM847X_H

