/*
 * em84device.c: Implementation of the SigmaDesign EM847x Device
 *
 * See the README file for copyright information and how to reach the author.
 *
 * $Id$
 */

#include "em84device.h"
#include "em84osd.h"

// --- cStreamDecoder -----------------------------------------------------------

cStreamDecoder::cStreamDecoder(cEm847x *Em847x)
{
    em847x = Em847x;
    gotBufferReserve = false;
    firstPacket=0;
}



cStreamDecoder::~cStreamDecoder() {
}

// --- AUDIO --------------------------------------------------------------------------
cAudioStreamDecoder::cAudioStreamDecoder(cEm847x *Em847x)
    : cStreamDecoder(Em847x)
{
    ringBuffer=new cRingBufferLinear(512*1024,2048,true);
//    Start(); // starte thread
}    

int cAudioStreamDecoder::Write(uchar *Data, int Length) 
{
    uint8_t *inbuf_ptr;                                 
    inbuf_ptr = (uint8_t *)Data;
    int size=Length;
    int pes_start=0, es_len=0, es_start=0;
    RMuint64 pts=0,dts=0;
//???    int consumed=0;
                if((Data[pes_start+0]==0x00)&&(Data[pes_start+1]==0x00)&&(Data[pes_start+2]==0x01))
                {
                        es_len=Data[pes_start+4]*256+Data[pes_start+5]+6-(9+Data[pes_start+8]);
                        es_start=pes_start+9+Data[pes_start+8];
                        if((es_len+es_start)<=size)
                        {
                                if((Data[pes_start+6]&0xC0)==0x80)
                                {
                                        if((Data[pes_start+7]&0xC0)==0x80)
                                        {
                                                firstPacket++;
                                                if (firstPacket>1) firstPacket=8;
                                                pts  = ((Data[pes_start+9] >> 1) & 0x07) << 29;
                                                pts |= Data[pes_start+10] << 21;
                                                pts |= (Data[pes_start+11] >> 1) << 14;
                                                pts |= Data[pes_start+12] << 6;
                                        }
                                        if ((Data[pes_start+3]>=0xC0)&&(Data[pes_start+3]<=0xCF))
                                        {
                                                //data consume = es_start+es_len;
                                                        em847x->PlayAudio(Data+es_start,es_len,pts,dts,0);
					}
                                // we have to restart from beginning
				}
                        }
                        // jump to the next Pes
                        // pes_start +=Data[pes_start+4]*256+Data[pes_start+5]+6;
                }
                else
                {
                        pes_start+=1;
                }
    return Length;
} 

void cAudioStreamDecoder::Action(){
	running=true;
	active=false;
    printf("Thread started\n");
}
void cAudioStreamDecoder::CleanUp() {
    ringBuffer->Clear();
    firstPacket=0;
    //active=false;
}
int cAudioStreamDecoder::Buffer() {
    return ringBuffer->Available();
}

cAudioStreamDecoder::~cAudioStreamDecoder()
{
    active = false;
  //  Cancel(3);
    delete(ringBuffer);
}

// --- VIDEO --------------------------------------------------------------------

cVideoStreamDecoder::cVideoStreamDecoder(cEm847x *Em847x)
    : cStreamDecoder(Em847x)
{
    ringBuffer=new cRingBufferLinear(1024*1024,4096,true);
    // Start(); // starte thread	
}    


int cVideoStreamDecoder::Write(uchar *Data, int Length) 
{
    int size=Length;
    int pes_start=0, es_len=0, es_start=0;
    RMuint64 pts=0,dts=0;
    int consumed=0;
    bool found=false;
		if((Data[pes_start+0]==0x00)&&(Data[pes_start+1]==0x00)&&(Data[pes_start+2]==0x01))
		{
			es_len=Data[pes_start+4]*256+Data[pes_start+5]+6-(9+Data[pes_start+8]);
			es_start=pes_start+9+Data[pes_start+8];
			if((es_len+es_start)<=size)
			{
				// We have enough data to decode ...
				// additional code to extract PTS and DTS
				if((Data[pes_start+6]&0xC0)==0x80)
				{
					// we have an MPEG2 bitstream
					if((Data[pes_start+7]&0xC0)==0x80)
					{
						firstPacket++;
						if (firstPacket>1) firstPacket=8;
						// Got the PTS
						pts  = ((Data[pes_start+9] >> 1) & 0x07) << 29;
						pts |= Data[pes_start+10] << 21;
						pts |= (Data[pes_start+11] >> 1) << 14;
						pts |= Data[pes_start+12] << 6;
					}
					else if((Data[pes_start+7]&0xC0)==0xC0)
					{
						firstPacket++;
						if (firstPacket>1) firstPacket=8;
						// Got PTS+DTS
						pts  = ((Data[pes_start+9] >> 1) & 0x07) << 29;
						pts |= Data[pes_start+10] << 21;
						pts |= (Data[pes_start+11] >> 1) << 14;
						pts |= Data[pes_start+12] << 6;
						pts |= Data[pes_start+13] >> 2;
						dts  = ((Data[pes_start+14] >> 1) & 0x07) << 29;
						dts |= Data[pes_start+15] << 21;
						dts |= (Data[pes_start+16] >> 1) << 14;
						dts |= Data[pes_start+17] << 6;
						dts |= Data[pes_start+18] >> 2;
					}
					if ((Data[pes_start+3]>=0xE0)&&(Data[pes_start+3]<=0xEF))
					{
						found=true;
						//data consume = es_start+es_len;
						consumed=es_start+es_len;
							em847x->PlayVideo(Data+es_start,es_len,pts,dts,0);
						//usleep(2);
					}
				}
			}
			// jump to the next Pes
			// pes_start +=Data[pes_start+4]*256+Data[pes_start+5]+6;
		}
  return Length;
}
void cVideoStreamDecoder::Action()
{
    // faut attendre un IFRAME a priori
    running=true;
    active=true;
    while(active) {
    }
    printf("Thread started\n");
}

void cVideoStreamDecoder::CleanUp() {
    ringBuffer->Clear();
    gotBufferReserve = false;
    firstPacket=0;
    //active=false;
}
int cVideoStreamDecoder::Buffer() {
    return ringBuffer->Available();
}
cVideoStreamDecoder::~cVideoStreamDecoder()
{
    active = false;
    Cancel(3);
    delete(ringBuffer);
}
// --- cEm84Device -----------------------------------------------------------

cEm84Device::cEm84Device()
{
  // printf("cEm84Device::cEm84Device\n");
  printf("Init real device new cEm847x()...");
  em847x = new cEm847x();
  printf("...Ok\n");
  state=0;
  startingPes=false;
  audio_size=0;
  video_size=0;
  printf("Init AudioStreamDecoder...\n");
    aout = new cAudioStreamDecoder(em847x);
  printf("...Ok\n");
  printf("Init VideoStreamDecoder...\n");
    vout = new cVideoStreamDecoder(em847x);
  printf("...Ok\n");
  // printf("cEm84Device::cEm84Device OK\n");
}
cEm84Device::~cEm84Device()
{
  // printf("m84Device::~cEm84Device\n");
  delete(em847x);
	printf("Halte Klassen an\n");
	while (decoding) {
	    usleep(1);
	}

	if (vout) {
	printf("Video ...");

//	    vout->Stop();
	    delete(vout);  
	printf(" gestoppt\n");

	}

	if (aout) {
	printf("Audio ...");

//	    aout->Stop();
	    delete(aout);
	printf(" gestoppt\n");
	    
	} 
	aout=vout=0;

  esyslog("SigmaDevice destroyed\n");
  // printf("m84Device::~cEm84Device OK\n");
}
void cEm84Device::SetVideoFormat(bool VideoFormat16_9) {
  em847x->SetVideoFormat(VideoFormat16_9);
}
cOsdBase *cEm84Device::NewOsd(int x, int y) {
  // printf("cEm84Device::NewOsd\n");
  return new cEm84Osd(x,y, em847x); 
  // printf("cEm84Device::NewOsd OK\n");
}
bool cEm84Device::HasDecoder(void) const { return true; }
bool cEm84Device::CanReplay(void) const { return true; }
bool cEm84Device::SetPlayMode(ePlayMode PlayMode) { 
  switch (PlayMode) {
    case pmNone:
  	printf("Stop Video Stream (pmNone)...\n");
	usleep(5);
	// while(vout->Buffer()>0) usleep(1);
	vout->CleanUp();
	aout->CleanUp();
	em847x->StopVideo();
    break;
    case pmAudioVideo:
  	printf("Start Video Stream (pmAudioVideo)...\n");
	vout->CleanUp();
	aout->CleanUp();
	// vout->Buffer();
	em847x->StartVideo();
	printf("-------------New Video Start--------------\n");
	state=0;
    break;
    case pmAudioOnlyBlack:
  printf("setting Playmode pmAudioOnlyBlack...\n");
	// vout->CleanUp();
	// em847x->StartVideo();
    break;
    case pmAudioOnly:
	// vout->CleanUp();
	// em847x->StartVideo();
  printf("setting Playmode pmAudioOnly...\n");
    break;
    case pmExtern_THIS_SHOULD_BE_AVOIDED:
  printf("setting Playmode pmExtern_THIS_SHOULD_BE_AVOIDED...\n");
    break;
  }
  playMode = PlayMode;
  return true;
}
void cEm84Device::TrickSpeed(int Speed) { 
	esyslog("Trickspeed not implemented yet...\n"); 
}
void cEm84Device::Clear(void) { 
	em847x->StopVideo();
	vout->CleanUp();
	aout->CleanUp();
	em847x->StartVideo();
	// em847x->StartVideo();
	printf("Clear device called\n");
}
void cEm84Device::Play(void) { 
esyslog("Play not implemented yet...\n"); 
}
void cEm84Device::Freeze(void) { 
	// em847x->PauseVideo();
	printf("Freeze video\n"); 
}

void cEm84Device::Mute(void) { esyslog("Mute not implemented yet...\n"); }
void cEm84Device::StillPicture(const uchar *Data, int Length) { esyslog("StillPicture not implemented yet...\n"); }
bool cEm84Device::Poll(cPoller &Poller, int TimeoutMs) { return true; }
//int cEm84Device::ProvidesCa(int Ca) { return 0; }
//int cEm84Device::ProvidesCa(const Channel *Channel)  { return 0; }
int cEm84Device::PlayBlackFrame() 
{ 
  return 0; 
}
int cEm84Device::PlayVideo(const uchar *Data, int Length)
{
    uint8_t *inbuf_ptr;                                 
    inbuf_ptr = (uint8_t *)Data;
	int pes_start=0; 	// PES offset (PES start @ Data+pes_start
	int es_start=0;		// ES offset (ES start @ Data+es_start
	int es_len=0;		// Length of the ES
	if (Length>0) {
		while(pes_start<Length-8) {
			// Packet to dispatch
			if((Data[pes_start+0]==0x00)&&(Data[pes_start+1]==0x00)&&(Data[pes_start+2]==0x01)) {
				es_len=Data[pes_start+4]*256+Data[pes_start+5]+6-(9+Data[pes_start+8]);
				es_start=pes_start+9+Data[pes_start+8];
				if((es_len+es_start)<=Length)	{
					// we have the full Packet, lets dispatch it
					if((Data[pes_start+6]&0xC0)==0x80) {
						// got MPEG2 Packet
						if ((Data[pes_start+3]>=0xE0)&&(Data[pes_start+3]<=0xEF)) {
							// Video Packet
							// Write full PES in ringBuffer
							// Write must get the full data
							vout->Write(inbuf_ptr+pes_start,es_len+es_start-pes_start);
						}
						else if ((Data[pes_start+3]>=0xC0)&&(Data[pes_start+3]<=0xCF)) {
							// Audio Packet
							// Write must get the full data
							aout->Write(inbuf_ptr+pes_start,es_len+es_start-pes_start);
						}
						// Go to nest PES
						pes_start = es_start+es_len;
					}
					else {
						// not MPEG2, let's skip it
						pes_start = es_start+es_len;
					}
				}
				else
				{
					// end of Data unexpected
					// Dispatched everything before es_start offset so : es_start = Length
					Length = es_start;
					// We have to stop the loop
					pes_start = Length; // will make the while break
				}
			}
			else {
				// not PES, go to next
				pes_start = pes_start+1;
			}
		}
	}
	return Length;
/*	
    uint8_t *inbuf_ptr;                                 
    inbuf_ptr = (uint8_t *)Data;
    // il faut trier les packets ici pour pouvoir ensuite desynchronizer l'audio et la video...
    // si audio -> 
    return vout->Write(inbuf_ptr,Length);
*/
}
