/*
 * em84osd.c: Implementation of the SigmaDesign EM847x Device
 *
 * See the README file for copyright information and how to reach the author.
 *
 * $Id$
 */
#ifndef __EM84DEVICE_H
#define __EM84DEVICE_H
#include <getopt.h>
#include <stdlib.h>

#include <vdr/device.h>
#include <vdr/osdbase.h>
#include <vdr/tools.h>
#include <vdr/ringbuffer.h>
#include <vdr/plugin.h>

#include "em847x.h"

// Output device handler
class cStreamDecoder  {
private:
    int state;
protected:
    bool gotBufferReserve;
    bool videoActive;
    cEm847x *em847x;
    bool buffering;
    int firstPacket;
public:
    virtual int Write(uchar *Data, int Length) = 0;
    virtual void CleanUp(void) = 0;
    virtual int Buffer(void) = 0;

    cStreamDecoder(cEm847x *em847x);
    virtual ~cStreamDecoder();
    void SyncPTS(uint64_t spts);
};


class cAudioStreamDecoder : public cStreamDecoder , cThread{
private:
    cRingBufferLinear *ringBuffer;
    bool active;
    bool running;
protected:
    virtual void Action(void);
public:
    cAudioStreamDecoder(cEm847x *em847x);
    ~cAudioStreamDecoder();
    virtual int Write(uchar *Data, int Length);
    virtual void CleanUp(void);
    virtual int Buffer(void);

};

class cVideoStreamDecoder : public cStreamDecoder , cThread {
private:
    cRingBufferLinear *ringBuffer;
    bool active;
    bool running;
    bool videoStart;
protected:
    virtual void Action(void);
public:
    cVideoStreamDecoder(cEm847x *em847x);
    ~cVideoStreamDecoder();
    virtual int Write(uchar *Data, int Length);
    virtual void CleanUp(void);
    virtual int Buffer(void);
};
// --- cEm84Device --------------------------------------------------

class cEm84Device : public cDevice {
  friend class cEm84Osd;
private:
  cEm847x *em847x;
    int  state, payload, streamtype;
    uint8_t headerLength;
    unsigned char header[0xFF];     
    unsigned int syncword;
    unsigned char * hdr_ptr;
    uint64_t commonPTS;
    bool running;
    bool decoding;
    bool validPTS;
    bool startingPes;
    bool firstPacket;
    int cpt,video_size,audio_size;
    cStreamDecoder *vout, *aout;

protected:
    ePlayMode playMode;
public:

  cEm84Device(void);
  virtual void SetVideoFormat(bool VideoFormat16_9);
  virtual bool HasDecoder(void) const;
  virtual bool CanReplay(void) const;
  virtual bool SetPlayMode(ePlayMode PlayMode);
  virtual void TrickSpeed(int Speed);
  virtual void Clear(void);
  virtual void Play(void);
  virtual void Freeze(void);
  virtual void Mute(void);
  virtual void StillPicture(const uchar *Data, int Length);
  virtual bool Poll(cPoller &Poller, int TimeoutMs = 0);
  virtual int PlayVideo(const uchar *Data, int Length);
  virtual int PlayBlackFrame(void);
  int ProvidesCa(int Ca);
  virtual cOsdBase *NewOsd(int x, int y);
  virtual ~cEm84Device();

};
#endif //__EM84DEVICE_H
