/*
 * em84osd.c: Implementation of the SigmaDesign EM847x On Screen Display
 *
 * See the README file for copyright information and how to reach the author.
 *
 * $Id$
 */

#include "em84osd.h"
// const cEm84Device *cEm84Osd::em84Device = NULL;

 cEm84Osd::cEm84Osd( int x, int y, cEm847x *Em847x)
  :cOsdBase(x, y) {
  xOfs=x-30;
  yOfs=y+10;
  em847x = Em847x;
  em847x->SetOsdOn(1);
  osd = (unsigned char *)malloc(sizeof(unsigned char)*720*576);
  lastUpdate = 0;
  for (int i = 0; i < MAXNUMWINDOWS; i++) {
    layer[i]=0;
  }
  OSDpresent=true;
}
cEm84Osd::~cEm84Osd() { 
  em847x->SetOsdOn(0);
  memset(osd,0xFF,720*576*sizeof(unsigned char));
  em847x->DrawOsd(osd);
  OSDpresent=false;
  free(osd);
  if(em847x) { em847x = 0; }
}

bool cEm84Osd::OpenWindow(cWindow *Window) {
    layer[Window->Handle()]= new cWindowLayer(Window->X0()+xOfs, Window->Y0()+yOfs,Window->Width(), Window->Height());
    /*
        for (int colorIndex = 0; colorIndex <MAXNUMCOLORS;colorIndex++) {
          const eDvbColor allColors = Window->GetColor(colorIndex);
	  em847x->SetOsdPalette((colorIndex+1+(MAXNUMCOLORS*Window->Handle())),((allColors >> 0x00) & 0x000000ff),((allColors >> 0x08) & 0x000000ff),((allColors >> 0x10) & 0x000000ff),((allColors >> 0x18) & 0x000000ff));
        }
    */
    // layer[Window->Handle()]->Render(Window);
    // FullOSDRefresh();
    return true;
}
void cEm84Osd::CommitWindow(cWindow *Window) {
    layer[Window->Handle()]->Render(Window);
    // printf("Window %d , Bpp = %d\n",Window->Handle(),Window->Bpp());
    int nbColors = 1 << Window->Bpp();
        for (int colorIndex = 0; colorIndex <nbColors;colorIndex++) {
          const eDvbColor allColors = Window->GetColor(colorIndex);
	  em847x->SetOsdPalette((colorIndex+1+(nbColors*Window->Handle())),((allColors >> 0x00) & 0x000000ff),((allColors >> 0x08) & 0x000000ff),((allColors >> 0x10) & 0x000000ff),((allColors >> 0x18) & 0x000000ff));
          // printf("%x ",allColors);
        }
    // printf("\n");
    /*
    int firstColorIndex = -1;
    int lastColorIndex = -1;
    const eDvbColor *modifiedColors = Window->NewColors(firstColorIndex, lastColorIndex);
    if (modifiedColors)
    {
      do
      {
        for (int colorIndex = firstColorIndex ; colorIndex <= lastColorIndex ; colorIndex++, modifiedColors++)
	{
          em847x->SetOsdPalette(colorIndex,(*modifiedColors & 0x000000ff) >> 0x00,(*modifiedColors & 0x0000ff00) >> 0x08,(*modifiedColors & 0x00ff0000) >> 0x10,(*modifiedColors & 0xff000000) >> 0x18);
        }
      }
      while (0 != (modifiedColors = Window->NewColors(firstColorIndex, lastColorIndex)));
    }
    */
    FullOSDRefresh();
}
void cEm84Osd::ShowWindow(cWindow *Window) {
  layer[Window->Handle()]->visible=true;
  layer[Window->Handle()]->Render(Window);
  FullOSDRefresh();
}
void cEm84Osd::HideWindow(cWindow *Window, bool Hide) {
  layer[Window->Handle()]->visible=false;
  layer[Window->Handle()]->Render(Window);
  FullOSDRefresh();
}
void cEm84Osd::MoveWindow(cWindow *Window, int x, int y) {
  layer[Window->Handle()]->Move(x,y);
  FullOSDRefresh();
}
void cEm84Osd::CloseWindow(cWindow *Window) {
    delete (layer[Window->Handle()]);
    FullOSDRefresh();
}
void cEm84Osd::FullOSDRefresh(void) {
  if (OSDpresent) {
      for (int i = MAXNUMWINDOWS-1; i >= 0; i--) {
        if(layer[i] && (layer[i]->visible==true))
        {
	  for (int yb = 0; yb <layer[i]->H();yb++)
	  {
            const int xosd = layer[i]->X()*sizeof(unsigned char);
            const int yosd = (yb+layer[i]->Y())*sizeof(unsigned char);
	    const int lenbmp = layer[i]->W()*sizeof(unsigned char);
	    const int ybmp = yb*sizeof(unsigned char)*lenbmp;
	    memcpy(osd+(xosd+yosd*720),layer[i]->Bitmap+ybmp,lenbmp);
	  }
        }
      }
  }
  em847x->DrawOsd(osd);
}
// -- cWindowLayer ----------------------------------------------------
cWindowLayer::cWindowLayer(int X, int Y, int W, int H) {
    left=X;
    top=Y;
    width=W;
    height=H;
    visible=false;
    Bitmap = (unsigned char *)malloc(sizeof(unsigned char)*width*height);
}
cWindowLayer::~cWindowLayer() {
    free(Bitmap);
}
void cWindowLayer::Move(int x, int y) {
    left=x;
    top=y;
}
unsigned char cWindowLayer::Data(int x, int y) {
    return Bitmap[y * width * sizeof(unsigned char) + x];
}
void cWindowLayer::Render(cWindow *Window) {
    int i = Window->Handle();
    int nbColors = 1 << Window->Bpp();
    for (int yp = 0; yp < height; yp++) {
	for (int ix = 0; ix < width; ix++) {
            Bitmap[yp * width * sizeof(unsigned char) + ix] = (unsigned char) (*Window->Data(ix,yp) +1+ i*nbColors);
            // Bitmap[yp * width * sizeof(unsigned char) + ix] = (unsigned char) (*Window->Data(ix,yp) );
	    // Bitmap[yp * width * sizeof(unsigned char) + ix] = Bitmap[yp * width * sizeof(unsigned char) + ix] + 1 + i*MAXNUMCOLORS;
	}
    }
}

